/* -*-c-*-
 *
 * $Id: anag.h,v 1.1 2001/02/04 17:14:42 mdw Exp $
 *
 * External definitions for Anag
 *
 * (c) 2001 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Anag: a simple wordgame helper.
 *
 * Anag is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Anag is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Anag; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: anag.h,v $
 * Revision 1.1  2001/02/04 17:14:42  mdw
 * Initial checkin
 *
 */

#ifndef ANAG_H
#define ANAG_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*----- Data structures ---------------------------------------------------*/

typedef struct node {
  int (*func)(struct node */*n*/, const char */*p*/, size_t /*sz*/);
} node;

typedef struct dstr {
  char *buf;
  size_t len;
  size_t sz;
} dstr;

#define DSTR_INIT { 0, 0, 0 }

/*----- Node types --------------------------------------------------------*/

extern node *anagram(const char *const */*av*/);
extern node *subgram(const char *const */*av*/);
extern node *wildcard(const char *const */*av*/);
extern node *trackword(const char *const */*av*/);

/*----- Error reporting ---------------------------------------------------*/

/* --- @ego@ --- *
 *
 * Arguments:	@const char *p@ = pointer to program name
 *
 * Returns:	---
 *
 * Use:		Stores what the program's name is.
 */

extern void ego(const char */*p*/);

/* --- @pquis@ --- *
 *
 * Arguments:	@FILE *fp@ = output stream to write on
 *		@const char *p@ = pointer to string to write
 *
 * Returns:	Zero if everything worked, EOF if not.
 *
 * Use:		Writes the string @p@ to the output stream @fp@.  Occurrences
 *		of the character `$' in @p@ are replaced by the program name
 *		as reported by @quis@.  A `$$' is replaced by a single `$'
 *		sign.
 */

extern int pquis(FILE */*fp*/, const char */*p*/);

/* --- @die@ --- *
 *
 * Arguments:	@const char *f@ = a @printf@-style format string
 *		@...@ = other arguments
 *
 * Returns:	Never.
 *
 * Use:		Reports an error and exits.
 */

extern void die(const char */*f*/, ...);

/*----- Memory allocation -------------------------------------------------*/

/* --- @xmalloc@ --- *
 *
 * Arguments:	@size_t sz@ = size of block to allocate
 *
 * Returns:	Pointer to allocated block.
 *
 * Use:		Allocates memory.  If there's not enough memory, the
 *		program exits.
 */

extern void *xmalloc(size_t /*sz*/);

/* --- @xrealloc@ --- *
 *
 * Arguments:	@void *p@ = a pointer to allocated memory
 *		@size_t sz@ = new size of block wanted
 *
 * Returns:	Pointer to resized block.
 *
 * Use:		Resizes an allocated block.  If there's not enough memory,
 *		the program exits.
 */

extern void *xrealloc(void */*p*/, size_t /*sz*/);

/*----- Dynamic string handling -------------------------------------------*/

/* --- @dstr_destroy@ --- *
 *
 * Arguments:	@dstr *d@ = pointer to a dynamic string block
 *
 * Returns:	---
 *
 * Use:		Reclaims the space used by a dynamic string.
 */

extern void dstr_destroy(dstr */*d*/);

/* --- @dstr_reset@ --- *
 *
 * Arguments:	@dstr *d@ = pointer to a dynamic string block
 *
 * Returns:	---
 *
 * Use:		Resets a string so that new data gets put at the beginning.
 */

extern void dstr_reset(dstr */*d*/);

/* --- @dstr_ensure@ --- *
 *
 * Arguments:	@dstr *d@ = pointer to a dynamic string block
 *		@size_t sz@ = amount of free space to ensure
 *
 * Returns:	---
 *
 * Use:		Ensures that at least @sz@ bytes are available in the
 *		given string.
 */

extern void dstr_ensure(dstr */*d*/, size_t /*sz*/);

/* --- @dstr_putline@ --- *
 *
 * Arguments:	@dstr *d@ = pointer to a dynamic string block
 *		@FILE *fp@ = a stream to read from
 *
 * Returns:	The number of characters read into the buffer, or @EOF@ if
 *		end-of-file was reached before any characters were read.
 *
 * Use:		Appends the next line from the given input stream to the
 *		string.  A trailing newline is not added; a trailing null
 *		byte is appended, as for @dstr_putz@.
 */

extern int dstr_putline(dstr */*d*/, FILE */*fp*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
